// -*- C++ -*-

/* 
 * gtkaxis.cc 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gtkaxis.h"

Gtk_Axis::Gtk_Axis(Axis_Base::alignment_t a) 
  : Axis_Base(a)
{
  set_events(get_events() | GDK_POINTER_MOTION_HINT_MASK);

  // This makes flicker hard to see, assuming the backing pixmap 
  // is mostly white.
  GtkStyle* style = gtk_style_copy(get_style()->gtkstyle);
  GdkColor color;
  gdk_color_parse("white",&color);
  style->bg[GTK_STATE_NORMAL] = color;
  set_style(style);
}

Gtk_Axis::~Gtk_Axis()
{
  
}

void 
Gtk_Axis::size_request_impl(GtkRequisition* r)
{
#ifdef GNOME_ENABLE_DEBUG
  g_print("Gtk_Axis::size_request\n");
#endif
  g_return_if_fail(r != 0);

  // arbitrary for now; later we compute at least one
  //  of the dimensions.
  if (horizontal()) {
    r->width = 0;
    r->height = 35; 
  }
  else {
    r->width = 35;
    r->height = 0;
  }
}

gint 
Gtk_Axis::button_press_event_impl(GdkEventButton* be)
{
#ifdef GNOME_ENABLE_DEBUG
  g_print("Gtk_Axis got button press\n");
#endif
  return FALSE;
}

gint 
Gtk_Axis::configure_event_impl(GdkEventConfigure* ce)
{
  // The order of events here is very important.
  new_pixmap(ce);
  // The *primary* drawable, screen(), is guaranteed to exist since
  // we're realized. Gdk_Draw can cope with a null secondary caused by
  // 0 allocation.
  set_drawable(screen());   
  set_secondary_drawable(buffer());

  redraw();

  return TRUE;
}

gint 
Gtk_Axis::motion_notify_event_impl(GdkEventMotion* me)
{
  gint x,y;
  GdkModifierType state;

  if (me->is_hint)
    gdk_window_get_pointer (me->window, &x, &y, &state);
  else
    {
      x = (gint) me->x;
      y = (gint) me->y;
      state = (GdkModifierType) me->state;
    }

#ifdef GNOME_ENABLE_DEBUG
  g_print("Gtk_Axis::motion_notify_event_impl: %d,%d\n", x,y);
#endif

  // does nothing; haven't decided what it should do.
  if (state & GDK_BUTTON1_MASK) {

    return FALSE;
  }

  return FALSE;
}

void
Gtk_Axis::redraw()
{
  set_drawing_target(this);

  //  gdk_window_clear(get_window());

  gc_save();
  // Eventually the user can set this.
  gc()->set_foreground("white");
  
  // should be done by setting widget background
  draw_rectangle(true,0,0,Gdk_Draw::width(),Gdk_Draw::height());

  gc_restore();
}




