// -*- C++ -*-

/* 
 * gtkplot.cc 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <algo.h>

#include "gtkplot.h"

Gtk_Plot::Gtk_Plot()
{
  //  rubberband_.lock_width(40);
  //  rubberband_.lock_height(-50);
  connect_to_method(rubberband_.snapped, this, 
		    &Gtk_Plot::rubberband_snapped);
  connect_to_method(rubberband_.changed, this, 
		    &Gtk_Plot::rubberband_changed);

  set_events(get_events() | GDK_POINTER_MOTION_HINT_MASK);

  // This makes flicker hard to see, assuming the backing pixmap 
  // is mostly white.
  GtkStyle* style = gtk_style_copy(get_style()->gtkstyle);
  GdkColor color;
  gdk_color_parse("white",&color);
  style->bg[GTK_STATE_NORMAL] = color;
  set_style(style);
}

Gtk_Plot::~Gtk_Plot()
{
  
}

gint 
Gtk_Plot::button_press_event_impl(GdkEventButton* be)
{
  g_print("Gtk_Plot got button press\n");
  if (be->button == 1) {
    rubberband_.start(this, this, be);
    return TRUE;
  }
  else return FALSE;
}

gint 
Gtk_Plot::configure_event_impl(GdkEventConfigure* ce)
{
  // The order of events here is very important.
  new_pixmap(ce);
  // The *primary* drawable, screen(), is guaranteed to exist since
  // we're realized. Gdk_Draw can cope with a null secondary caused by
  // 0 allocation.
  set_drawable(screen());
  set_secondary_drawable(buffer());

  redraw();

  return TRUE;
}

gint 
Gtk_Plot::motion_notify_event_impl(GdkEventMotion* me)
{
  gint x,y;
  GdkModifierType state;

  if (me->is_hint)
    gdk_window_get_pointer (me->window, &x, &y, &state);
  else
    {
      x = (gint) me->x;
      y = (gint) me->y;
      state = (GdkModifierType) me->state;
    }

  if (state & GDK_BUTTON1_MASK) {
    // We're doing rubberband
    return FALSE;
  }
  else {
    gdouble cx, cy, width, height;

    cx = x_transform()->inverse(x);
    cy = y_transform()->inverse(y);

    // FIXME requires Gtk--Draw change
    width = x_max() - x_min();
    height = x_max() - x_min();

    coordinates(cx,cy,width,height);
    
    // this is just for a status display, so don't block other handlers.
    return FALSE;
  }
}

void
Gtk_Plot::redraw()
{
  set_drawing_target(this);
  
  //  gdk_window_clear(get_window());

  gc_save();
  // Eventually the user can set this.
  gc()->set_foreground("white");
  
  // We have to explicitly draw the background, because of the 
  //  backing pixmap. The widget's style background is irrelevant.
  
  draw_rectangle(true,0,0,Gdk_Draw::width(),Gdk_Draw::height());

  gc_restore();
}

void 
Gtk_Plot::rubberband_snapped(gint x, gint y, gint w, gint h)
{
  gdouble cx0, cy0, cx1, cy1;

  cx0 = x_transform()->inverse(x);
  cy0 = y_transform()->inverse(y);
  cx1 = x_transform()->inverse(x + w);
  cy1 = y_transform()->inverse(y + h);

  // put them in order
  gdouble tmp;

  if (cx0 > cx1) {
    tmp = cx0;
    cx0 = cx1;
    cx1 = tmp;
  }

  if (cy0 > cy1) {
    tmp = cy0;
    cy0 = cy1;
    cy1 = tmp;
  }

  new_bounds_request(cx0,cx1,cy0,cy1);

  g_print("RB snapped cb in Gtk_Plot; new bounds X: %f...%f to Y %f...%f\n",
	  cx0,cx1,cy0,cy1);
}

void 
Gtk_Plot::rubberband_changed(gint x_origin, gint y_origin, 
			     gint x, gint y)
{
  gdouble cx_origin, cy_origin, cx, cy;

  cx_origin = x_transform()->inverse(x_origin);
  cy_origin = y_transform()->inverse(y_origin);
  cx        = x_transform()->inverse(x);
  cy        = y_transform()->inverse(y);

  rubberband_coordinates(cx_origin, cy_origin, cx, cy);
  //  g_print("RB changed cb in Gtk_Plot\n");
}

