// -*- C++ -*-

/* 
 * gtkplot.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef GUPPI_GTKPLOT_H
#define GUPPI_GTKPLOT_H

#include <list>
#include <gtk--draw.h>
#include "plot_area.h"

#include "rubberband.h"

class Gtk_Plot : public Gtk_DrawingBuffer,
		 public Gdk_Draw,
		 public Plot_Area {
public:
  Gtk_Plot();
  virtual ~Gtk_Plot();

  // Current X, Y; Width, Height
  Signal4<gdouble,gdouble,gdouble,gdouble> coordinates;
  // X origin, Y origin, xpos, ypos
  Signal4<gdouble,gdouble,gdouble,gdouble> rubberband_coordinates;

  // User has requested a new set of bounds.
  // xmin, xmax, ymin, ymax
  Signal4<gdouble,gdouble,gdouble,gdouble> new_bounds_request;

protected:

  virtual gint button_press_event_impl(GdkEventButton* be);
  virtual gint configure_event_impl(GdkEventConfigure* ce);
  virtual gint motion_notify_event_impl(GdkEventMotion* me);

  void redraw();

  RubberBand rubberband_;

  void rubberband_snapped(gint x, gint y, gint w, gint h);
  void rubberband_changed(gint x_origin, gint y_origin, 
			  gint x, gint y);

private:

};

#endif
