// This is -*- C++ -*-
// $Id: plot_data.h,v 1.1 1998/09/01 19:49:44 hp Exp $

/* 
 * plot_data.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_PLOT_DATA_H
#define _INC_PLOT_DATA_H

// A simple, uni-directional STL-style iterator
class Plot_Data_Const_Iterator {
public:
  Plot_Data_Const_Iterator(const double* c, size_t pos, int s) :
    current_(c), pos_(pos), stride_(s)
    { }

  Plot_Data_Const_Iterator(const Plot_Data_Const_Iterator& pdi) :
    current_(pdi.current_), pos_(pdi.pos_), stride_(pdi.stride_)
    { }

  double operator*() const { return *current_; }

  Plot_Data_Const_Iterator& operator++() {
    current_ = (double*)(((char*)current_)+stride_);
    ++pos_;
    return *this;
  }

  Plot_Data_Const_Iterator operator++(int) {
    Plot_Data_Const_Iterator save(*this);
    operator++();
    return save;
  }

  bool operator==(const Plot_Data_Const_Iterator& x) const {
    // Compare looks only at the value of the index, so comparing
    // iterators from different data objects can get strange...
    return x.pos_ == pos_;
  }

  bool operator!=(const Plot_Data_Const_Iterator& x) const {
    // Ditto...
    return x.pos_ != pos_;
  }


private:
  const double* current_;
  size_t pos_;
  int stride_;
};

class Plot_Data {
public:
  typedef Plot_Data_Const_Iterator const_iterator;

  Plot_Data(const double* a, size_t N) :
    base_(a), length_(N), stride_(sizeof(double)),
    begin_(base_,0,stride_), end_(base_,length_,stride_)
    { }

  Plot_Data(const double* a, size_t N, int s) :
    base_(a), length_(N), stride_(s),
    begin_(base_,0,stride_), end_(base_,length_,stride_)
    { }

  size_t size() const { return length_; }

  const const_iterator& begin() const { return begin_; }
  const const_iterator& end() const { return end_; }

private:
  const double* base_;
  size_t length_;
  int stride_;
  const_iterator begin_, end_;
};

#endif // _INC_PLOT_DATA_H

// $Id: plot_data.h,v 1.1 1998/09/01 19:49:44 hp Exp $
