// -*- C++ -*-

/* 
 * plotlayout.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef GUPPI_PLOTLAYOUT_H
#define GUPPI_PLOTLAYOUT_H

#include "plotlayout.h"
#include "gtkaxis.h"

class PlotLayoutImpl;

// Handles layout of plot/axis widgets, and their destruction. It "owns" 
// any widget you put in it.

// A "group" is one plot and 4 axes, one of each alignment.

class PlotLayout : public Gtk_Frame {
public:
  PlotLayout();
  ~PlotLayout();

  // blow everything away.
  void clear(); 

  // These calls destroy any existing plot or axis and expand the layout
  //  if necessary.
  void place_plot(Gtk_Widget* w, size_t row, size_t col);
  void place_axis(Gtk_Axis* w, size_t row, size_t col, Axis_Base::alignment_t a);

  // Destroy everything on a row or column and shift
  //  other rows/columns accordingly.
  void remove_row(size_t row);
  void remove_column(size_t col);
  
  // These don't shift other things, they just leave a blank.
  void erase_axis(size_t row, size_t col, Axis_Base::alignment_t a);
  void erase_plot(size_t row, size_t col);
  // kill all 4 axes at once.
  void erase_axes(size_t row, size_t col);
  // delete both axes and plots
  void erase_group(size_t row, size_t col);  
  
  // move stuff, overwriting if necessary
  void move_axis (size_t row1, size_t col1, size_t row2, size_t col2, 
		  Axis_Base::alignment_t a);
  void move_plot (size_t row1, size_t col1, size_t row2, size_t col2);
  void move_group(size_t row1, size_t col1, size_t row2, size_t col2);
  void swap_group(size_t row1, size_t col1, size_t row2, size_t col2);

  // Check on state
  bool plot_exists(size_t row, size_t col);
  bool axis_exists(size_t row, size_t col, Axis_Base::alignment_t a);
  bool group_exists(size_t row, size_t col);

  // per-plot title
  void set_title(size_t row, size_t col, const string & title);
  // whole-grid title.
  void set_title(const string & title);

  Gtk_Widget* get_plot(size_t row, size_t col);
  Gtk_Axis*   get_axis(size_t row, size_t col, Axis_Base::alignment_t a);

  size_t rows() const; 
  size_t columns() const; 

private:
  PlotLayoutImpl* impl_;
};

#endif





