// -*- C++ -*-

/* 
 * rubberband.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef GUPPI_RUBBERBAND_H
#define GUPPI_RUBBERBAND_H

#include <gtk--draw.h>

class RubberBand : public Gtk_Signal_Base {
public:
  // Where to draw, where to get events, button press event that started this
  RubberBand();
  virtual ~RubberBand();

  // These two can be safely called at any time;
  //  they'll be ignored if we've already started
  //  or stopped.
  void start(Gdk_Draw* draw, Gtk_Widget* w, GdkEventButton * be);
  void stop();

  // x, y, w, h - user released button to complete selection
  // The x,y here is always the top left
  Signal4<gint,gint,gint,gint> snapped;
  
  // x_origin, y_origin, x, y - used to set locking 
  //  on the fly as the user moves
  Signal4<gint,gint,gint,gint> changed;

  // Args here can be negative, to lock
  //  in the opposite direction.
  void lock_width(gint w);
  void lock_height(gint h);
  void unlock();

protected:
  void draw();
  void undraw();

  gint motion_notify_cb(GdkEventMotion* me);
  gint button_release_cb(GdkEventButton* be);
  gint button_press_cb(GdkEventButton* be);

private:
  gint x_origin_, y_origin_;
  gint x_, y_;
  Connection motion_cnxn_;
  Connection release_cnxn_;
  Connection press_cnxn_;
  guint32 time_;
  bool stretching_;
  Gdk_Draw* draw_;
  GdkRectangle last_;
  
  bool width_locked_, height_locked_;
  gint width_lock_, height_lock_;
};

#endif
